package wiki

import (
	//"encoding/json"
	"net/http"

	"github.com/Sirupsen/logrus"
	"github.com/dalu/wiki/wiki/storage"
	"github.com/dalu/wiki/wiki/template"
)

const DefaultMountPath = "/wiki/"

const (
	methodGET  = "GET"
	methodPOST = "POST"
)

type wikiHandler struct {
	s storage.Interface
	r template.Interface
	l *logrus.Logger
}

func NewWikiHandler(s storage.Interface, r template.Interface, l *logrus.Logger) *wikiHandler {
	l.Level = logrus.DebugLevel
	return &wikiHandler{
		s: s,
		r: r,
		l: l,
	}
}

func (h *wikiHandler) ServeHTTP(w http.ResponseWriter, r *http.Request) {
	switch r.Method {
	case methodGET:
		h.handleGET(w, r)
	case methodPOST:
		h.handlePOST(w, r)
	default:
		http.Error(w, http.StatusText(http.StatusMethodNotAllowed), http.StatusMethodNotAllowed)
	}
}

func (h *wikiHandler) handleGET(w http.ResponseWriter, r *http.Request) {
	action := r.URL.Query().Get("action")

	switch action {
	case "":
		h.l.Debug("show hit")
		h.show(w, r)
	case "edit":
		h.l.Debug("edit hit")
	case "new":
		h.l.Debug("new hit")
	case "revision":
		h.l.Debug("revision hit")
	case "revisions":
		h.l.Debug("revisions hit")
	}
}

func (h *wikiHandler) handlePOST(w http.ResponseWriter, r *http.Request) {}

// -----

func (h *wikiHandler) show(w http.ResponseWriter, r *http.Request) {
	term, e := h.s.GetTermBySlug(r.URL.Path)
	if e != nil {
		h.l.Error("wikihandler.show.GetTermBySlug:", e)
		return
	}
	if term.Redirect != "" {
		http.Redirect(w, r, DefaultMountPath+term.Redirect, http.StatusTemporaryRedirect)
		return
	}
	if term.CurrentRevisionID == "" {
		http.Redirect(w, r, DefaultMountPath+r.URL.Path+"?action=new", http.StatusTemporaryRedirect)
		return
	}
	revision, e := h.s.GetRevisionByID(term.CurrentRevisionID)
	if e != nil {
		h.l.Error("wikihandler.show.GetRevisionByTermID:", e)
		return
	}

	ctx := make(map[string]interface{})
	ctx["term"] = term
	ctx["revision"] = revision

	if e := h.r.Render(w, "wiki/show.html.twig", ctx); e != nil {
		h.l.Error("wikiHandler.show.Renderer.write:", e)
	}

	// temporary
	/*
	if e := json.NewEncoder(w).Encode(revision); e != nil {
		h.l.Error("wikihandler.show.NewEncoder:", e)
		http.Error(w, e.Error(), http.StatusInternalServerError)
	}
	*/
}
